import json
import importlib
from . import file_mod

outfuncs = {}

# 注册输出格式
def register(name=""):
    if name == "":
        def func(tf):
            outfuncs[tf.__name__] = tf
            return tf
    else:
        def func(tf):
            outfuncs[name] = tf
            return tf
    return func

@register()
def default(hlist: list):
    # 计算文件路径的最大长度
    max_file_length = max(len(item["file"]) for item in hlist)
    
    # 加上4个空格
    max_file_length += 4
    
    output = [f"""{"File":<{max_file_length}}Hash"""]
    for i in hlist:
        output += [f"""{i["file"]:<{max_file_length}}{i["hash"]}"""]
    return "\n".join(output)


@register()
def md(hlist:list):
    output = ["|File|Hash|", "|-|-|"]
    for i in hlist:
        output += [f"""|{i["file"]}|{i["hash"]}|"""]
    return "\n".join(output)

@register()
def csv(hlist:list):
    output = ["File,Hash"]
    for i in hlist:
        output += [f"""{i["file"]},{i["hash"]}"""]
    return "\n".join(output)

@register("json")
def thejson(hlist:list):
    return json.dumps(hlist, indent=4)

def chmod(hlist:list,modname:str="default"):
    if modname not in outfuncs: 
        full = modname.split('|')
        mod=file_mod.get_function_object(full[0],full[1])
    else: mod=outfuncs[modname]
    return mod(hlist)



from . import outs
from .hashs import hash
from .enumerate import get_file_or_folder_contents_list
from .console import Manager
import time

manager = Manager()
def shell(args):
    start_time = time.time()
    manager.print("开始枚举文件...")
    files = get_file_or_folder_contents_list(args["file"],args["reverse"])
    manager.print(f"共找到 {len(files)} 个文件")
    manager.print("开始计算...")
    hash_list = hash(files,args["method"],args["length"],manager.print)
    manager.print("开始格式化...")
    out = outs.chmod(hash_list,args["outmod"])
    if args["outfile"] != "NO":
        with open(args["outfile"],'w') as f:
            f.write(out)
        print(f"已输出到文件： {args['outfile']}")
        print(f"耗时：{time.time() - start_time} 秒")
    else:
        manager.clear()
        print(out)


import sys

class Manager:
    def __init__(self):
        self.output_lines = 0  # 记录输出的行数

    def print(self, *text, end="\n"):
        """打印文本并统计换行符"""
        # 计算文本中包含的换行符数量
        if len(text) == 0:
            outtext = str(end)
        else:
            outtext = str(text[0])
            for t in text[1:]:
                outtext += " " + str(t)
            outtext += str(end)

        self.output_lines += outtext.count("\n")  # 每个换行符算一行
        sys.stdout.write(outtext)
        sys.stdout.flush()

    def clear(self):
        """清除所有输出行"""
        # 将光标移动到顶部
        sys.stdout.write("\r")
        # 清除所有输出行
        for _ in range(self.output_lines + 1):
            sys.stdout.write("\033[K\033[F")  # 光标向上移动一行并且清除
        sys.stdout.write("\n")  # 确保光标移动到下一行
        sys.stdout.flush()
        self.output_lines = 0

if __name__ == "__main__":
    import time
    manager = Manager()
    manager.print("Hello, World!\n666")
    manager.print("This is a test.")
    time.sleep(2)
    manager.clear()
        


import argparse
from.shell import shell
from . import about
import haaaash
gui = None
if haaaash.the == 'haaaash-expansion':
    from . import gui

def main():
    parser = argparse.ArgumentParser(
                        prog='haaaash',
                        description='详细解析：',
                        epilog='By Gudupao (MIT License)')
    parser.add_argument('file',help='文件(夹)路径（默认为当前目录）',nargs='*',default='.')
    parser.add_argument('-m','--method',help='哈希方法（默认为 sha256）',default='sha256')
    parser.add_argument('-l','--length',help='哈希长度（算法为 shake_128 shake_256 时）',type=int,default=20)
    parser.add_argument('-o','--outmod',help='输出模式',type=str,default="default")
    parser.add_argument('-f','--outfile',help='输出文件',type=str,default="NO")
    parser.add_argument('-r','--reverse',help='是否反向输出（深层路径在前）',action='store_true')
    parser.add_argument('-a','--about',help='关于',action='store_true')
    parser.add_argument('-g','--gui',help='启动图形界面',action='store_true')
    parser.add_argument('--TEST',help='测试模式，激活高级版',action='store_true')
    
    args = parser.parse_args()
    
    if args.about:
        about.main()
        return
        
    if args.gui:
        global gui
        if haaaash.the == 'haaaash-expansion':
            if gui:
                gui.main()
            else:
                print("GUI 模块加载失败")
        elif args.TEST:
            try:
                from . import gui
                gui.main()
            except ImportError:
                print("GUI 模块加载失败")
        else:
            print("请安装 haaaash-expansion 扩展版")
        return

    args = args.__dict__
    shell(args)

if __name__ == '__main__':
    main()


import importlib.util
import os

def get_function_object(file_path, function_name):
    # 获取文件的绝对路径
    file_path = os.path.abspath(file_path)

    # 获取文件名和模块名
    module_name = os.path.splitext(os.path.basename(file_path))[0]

    # 动态加载模块
    spec = importlib.util.spec_from_file_location(module_name, file_path)
    module = importlib.util.module_from_spec(spec)
    spec.loader.exec_module(module)

    # 获取函数对象
    if hasattr(module, function_name):
        return getattr(module, function_name)
    else:
        return None  # 如果没有找到指定的函数，返回None

if __name__ == "__main__":
    # 使用示例
    full = 'D:/python/Haaaash/example.test.py|my_function'
    full = full.split('|')
    file_path = full[0]
    function_name = full[1]
    func = get_function_object(file_path, function_name)

    if func:
        print(f"成功获取函数对象: {func}")
        # 可以调用这个函数
        func()  # 假设函数没有参数
    else:
        print(f"未找到函数 {function_name}")



from setuptools import setup, find_packages

filepath = 'README.md'

setup(
    name='haaaash',
    version='0.8.5',
    author='Gudupao',
    author_email='official@gudupao.top',
    description='hash 批量计算器',
    long_description=open(filepath, encoding='utf-8').read(),
    long_description_content_type='text/markdown',
    packages=find_packages(),
    install_requires=[],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
        'Programming Language :: Python :: 3.13',
    
    ],
    entry_points={
        'console_scripts': [
            'haaaash=haaaash.__main__:main',
        ],
    },
)


import os

def get_file_or_folder_contents(path,r:bool=False):
    # 判断路径是否存在
    if not os.path.exists(path):
        raise ValueError(f"路径 {path} 不存在")
    
    # 判断是文件还是文件夹
    if os.path.isfile(path):
        return [path]  # 如果是文件，返回原路径作为列表
    elif os.path.isdir(path):
        file_paths = []
        # 遍历文件夹中的文件
        for root, dirs, files in os.walk(path):
            for file in files:
                file_paths.append(os.path.join(root, file))
        if r:
            return file_paths[::-1]
        else:
            return file_paths
    else:
        raise ValueError(f"路径 {path} 既不是文件也不是文件夹")

# 针对路径列表
def get_file_or_folder_contents_list(path_list:list,r:bool=False):
    output = []
    for i in path_list:
        output += get_file_or_folder_contents(i,r)
        
    return output
